LIBRARY	"CharLS"
EXPORTS
    JpegLsEncode
    JpegLsDecode
    JpegLsDecodeRect        
    JpegLsReadHeader  
    JpegLsVerifyEncode
    JpegLsEncodeStream
    JpegLsDecodeStream
    JpegLsReadHeaderStream